︠916fccdf-9628-4b8c-9e96-39b6eaa2db4aai︠
%hide
%auto

def html_table(table, header=[],style='',header_style='',cell_style=''):
    s = "<html><table style='%s'>"%style
    if len(header)>0:
        s+= "<tr><th style='%s'>"%header_style
        s+= ("</th><th style='%s'>"%header_style).join(str(i) for i in header)
        s+= "</th></tr>"
    for row_index in range(len(table)):
        row = table[row_index]
        if isinstance(cell_style,list):
            cstyle=cell_style[row_index]
        else:
            cstyle=cell_style
        s+= "<tr><td style='%s'>"%cstyle
        s+= ("</td><td style='%s'>"%cstyle).join(str(i) for i in row)
        s+= "</td></tr>"
    s+= "</table>"
    return html(s)
︡5503c439-62b9-4053-a580-e1313a6104f6︡︡
︠e377c9ce-5a42-4063-8816-8389bbd02fccai︠
%hide
%auto

def illustrate_test(f, fvals=(-5,5), tvals=(-3,3), line_thickness=3,surface_color='blue', line_color=['red','green'], cell_style='border-style: solid; border-width: 1px;'):
    """
    Return a table of critical points, eigenvalues, and eigenvectors, plus a 3D graph illustrating the curvature at each critical point.
    
    INPUTS:
      f -- a function.  Usually this should be a callable function, defined like f(x,y)=x^2+y^2, for example.
      fvals (default (-5,5)) -- minimum and maximum values for the function plot
      tvals (default (-3,3)) -- minimum and maximum values for the parameter in the curves at each critical point
      line_thickness (default 3) -- how thick the lines are
      surface_color (default 'blue') -- color of the graph
      line_color (default ['red', 'green'] -- Color of lines at each critical point.  Colors will alternate.
      cell_style (default 'border-style: solid; border-width: 1px;') -- CSS style for the cells of the table

    EXAMPLES:
        sage: var('x,y')
        sage: f(x,y)=-x^2+y^2
        sage: table, graph=illustrate_test(f)
        sage: print table
        sage: show(graph)
        
        sage: var('x,y')
        sage: f(x,y)=x*y*exp((-x^2-y^2)/3)
        sage: table, graph=illustrate_test(f)
        sage: print table
        sage: show(graph)
    """
    t=var('t')
    tmin, tmax = tvals
    fmin, fmax = fvals
    f_parametric = vector([x,y,f(x,y)])
    picture = plot3d(f,(x,fmin,fmax),(y,fmin,fmax), opacity=0.3, surface_color=surface_color)
    hessian=f(x,y).hessian()
    assume(x,'real')
    assume(y,'real')
    solns=solve([eqn(x,y)==0 for eqn in f.gradient()],[x,y], solution_dict=True)
    
    table=[]
    table_cell_styles=[]
    pictures=[]
    for solution in [s for s in solns if s[x] in RR and s[y] in RR]:
        soln = dict([(key,RDF(val)) for key,val in solution.items()])
        critical_point = f_parametric(soln)
        hessian_at_point = hessian(soln).change_ring(RDF)
        evals,evecs=hessian_at_point.eigenvectors_right()
        picture += point3d(critical_point.list(), color='green', size=10)
        for eval,evec in zip(evals,evecs.columns()):
            direction_plot = evec.plot(width=line_thickness,color=line_color[0]).plot3d().translate(critical_point)    
            curve_formula = f_parametric(x=soln['x']+t*evec[0],y=soln['y']+t*evec[1])
            curve = parametric_plot(curve_formula.list(), (t,tmin,tmax),color=line_color[0],thickness=line_thickness)
            picture+=curve+direction_plot
            pictures += [curve]
            table.append([critical_point, eval, evec])
            table_cell_styles.append(cell_style+'color: %s;'%line_color[0])
    
            # Rotate the colors in the list
            line_color.append(line_color.pop(0))
    
    return html_table(table,header=['Critical Point', 'Eigenvalue', 'Eigenvector'], style='border-collapse: collapse;', cell_style=table_cell_styles), picture
︡45588f84-cf57-4287-b54c-813c2ade6dd1︡︡
︠c763c990-4064-4201-999b-e21d4ff383fbi︠
%html
<p><span class="typeset"><span class="scale"><span style="position: relative;"></span></span></span></p>

<h1>The 2nd Derivative test</h1>
<p>The second derivative test looks at the eigenvalues of the Hessian matrix evaluated at critical points.</p>
<p>The idea for this came from Ben Woodruff, who implemented a similar thing in Mathematica.</p>

︡cfb0a01d-27f7-432a-ba0b-877958243b13︡{"html": "<p><span class=\"typeset\"><span class=\"scale\"><span style=\"position: relative;\"></span></span></span></p>\n\n<h1>The 2nd Derivative test</h1>\n<p>The second derivative test looks at the eigenvalues of the Hessian matrix evaluated at critical points.</p>\n<p>The idea for this came from Ben Woodruff, who implemented a similar thing in Mathematica.</p>"}︡
︠8b01a1f2-616d-4197-bd9c-9b002d731eff︠
var('x,y')
︡59133291-919b-462a-8c5b-0f759a33a4e7︡{"stdout": "(x, y)"}︡
︠7fe68474-077f-4058-bdb5-b4f6a37323e2︠
f(x,y)=-x^2+y^2
table, graph=illustrate_test(f)
print table
show(graph)
︡dfa34a97-297f-4edc-a8a5-1f157a2d4cc3︡{"html": "<font color='black'><html><table style='border-collapse: collapse;'><tr><th style=''>Critical Point</th><th style=''>Eigenvalue</th><th style=''>Eigenvector</th></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(0.0, 0.0, 0.0)</td><td style='border-style: solid; border-width: 1px;color: red;'>-2.0</td><td style='border-style: solid; border-width: 1px;color: red;'>(1.0, 0.0)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(0.0, 0.0, 0.0)</td><td style='border-style: solid; border-width: 1px;color: green;'>2.0</td><td style='border-style: solid; border-width: 1px;color: green;'>(0.0, 1.0)</td></tr></table></font>"}︡
︠952d161e-895c-44bc-b21b-9ed24e360671︠
f(x,y)=x*y*exp((-x^2-y^2)/3)
table, graph=illustrate_test(f)
print table
show(graph)
︡b94dbc63-a4a9-4212-9e89-4401babad543︡{"html": "<font color='black'><html><table style='border-collapse: collapse;'><tr><th style=''>Critical Point</th><th style=''>Eigenvalue</th><th style=''>Eigenvector</th></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(0.0, 0.0, 0.0)</td><td style='border-style: solid; border-width: 1px;color: red;'>1.0</td><td style='border-style: solid; border-width: 1px;color: red;'>(0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(0.0, 0.0, 0.0)</td><td style='border-style: solid; border-width: 1px;color: green;'>-1.0</td><td style='border-style: solid; border-width: 1px;color: green;'>(-0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(-1.22474487139, -1.22474487139, 0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: red;'>-0.735758882343</td><td style='border-style: solid; border-width: 1px;color: red;'>(0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(-1.22474487139, -1.22474487139, 0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: green;'>-0.735758882343</td><td style='border-style: solid; border-width: 1px;color: green;'>(-0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(1.22474487139, -1.22474487139, -0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: red;'>0.735758882343</td><td style='border-style: solid; border-width: 1px;color: red;'>(0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(1.22474487139, -1.22474487139, -0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: green;'>0.735758882343</td><td style='border-style: solid; border-width: 1px;color: green;'>(-0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(-1.22474487139, 1.22474487139, -0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: red;'>0.735758882343</td><td style='border-style: solid; border-width: 1px;color: red;'>(0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(-1.22474487139, 1.22474487139, -0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: green;'>0.735758882343</td><td style='border-style: solid; border-width: 1px;color: green;'>(-0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: red;'>(1.22474487139, 1.22474487139, 0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: red;'>-0.735758882343</td><td style='border-style: solid; border-width: 1px;color: red;'>(0.707106781187, 0.707106781187)</td></tr><tr><td style='border-style: solid; border-width: 1px;color: green;'>(1.22474487139, 1.22474487139, 0.551819161757)</td><td style='border-style: solid; border-width: 1px;color: green;'>-0.735758882343</td><td style='border-style: solid; border-width: 1px;color: green;'>(-0.707106781187, 0.707106781187)</td></tr></table></font>"}︡
︠074e20c3-da6e-411c-bff6-0e68dcb0eb15︠
illustrate_test?
︡4df874ca-1f61-4e49-b6c0-6f96811d206b︡︡
